/*
 *    TTTTTTTTTTTTTT  EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *    TTTTTTTTTTTTTT  EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EEEEEEEEEE      OO          OO
 *          TT        EEEEEEEEEE      OO          OO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *          TT        EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *
 *                  L'mulateur Thomson TO8
 *
 *  Copyright (C) 1997-2001 Gilles Ftis, Eric Botcazou, Alexandre Pukall,
 *                          Jrmie Guillaume
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : linux/display.c
 *  Version    : 1.7.5
 *  Cr par   : Eric Botcazou octobre 1999
 *  Modifi par: Eric Botcazou 24/11/2003
 *
 *  Module d'interface avec le serveur X.
 */


#ifndef SCAN_DEPEND
   #include <stdio.h>
   #include <stdlib.h>
   #include <gdk/gdkx.h>
   #include <X11/Xlib.h>
   #include <X11/Xutil.h>
   #include <X11/keysym.h>
#endif

#include "linux/display.h"
#include "linux/graphic.h"
#include "linux/main.h"
#include "to8keys.h"
#include "to8.h"


Display *display;
int screen;
int mit_shm_enabled;
Window screen_win;

static int installed_pointer;
static Cursor null_cursor;
static int x11_to_dos[256];

#define KB_SIZE  100
static struct {
    int keysym;
    int keycode;
} keyconv[KB_SIZE]={ 	{XK_Escape, 		KEY_ESC},
			{XK_1, 			KEY_1  },
			{XK_2,			KEY_2  },
			{XK_3,	 		KEY_3  },
			{XK_4,	 		KEY_4  },
			{XK_5,	 		KEY_5  },
			{XK_6,	 		KEY_6  },
			{XK_7,	 		KEY_7  },
			{XK_8,			KEY_8  },
			{XK_9,			KEY_9  },
			{XK_0,	 		KEY_0  },
			{XK_parenright,		KEY_MINUS },
			{XK_equal, 		KEY_EQUALS },
			{XK_BackSpace, 		KEY_BACKSPACE },
			{XK_Tab, 		KEY_TAB },
			{XK_A,	 		KEY_Q },
			{XK_Z,	 		KEY_W },
			{XK_E,			KEY_E },
			{XK_R,			KEY_R },
			{XK_T,	 		KEY_T },
			{XK_Y,	 		KEY_Y },
			{XK_U,	 		KEY_U },
			{XK_I,	 		KEY_I },
			{XK_O, 			KEY_O },
			{XK_P, 			KEY_P },
			{XK_dead_circumflex,    KEY_OPENBRACE },
			{XK_dollar, 		KEY_CLOSEBRACE },
			{XK_Return, 		KEY_ENTER },
			{XK_Control_L, 		KEY_LCONTROL },
			{XK_Q,	 		KEY_A },
			{XK_S,	 		KEY_S },
			{XK_D,	 		KEY_D },
			{XK_F,	 		KEY_F },
			{XK_G,	 		KEY_G },
			{XK_H,			KEY_H },
			{XK_J, 			KEY_J },
			{XK_K, 			KEY_K },
			{XK_L,			KEY_L },
			{XK_M, 			KEY_COLON },
			{XK_percent, 		KEY_QUOTE },
			{XK_twosuperior, 	KEY_TILDE },
			{XK_Shift_L, 		KEY_LSHIFT },
			{XK_asterisk, 		KEY_ASTERISK },
			{XK_W, 			KEY_Z },
			{XK_X, 			KEY_X },
			{XK_C, 			KEY_C },
			{XK_V, 			KEY_V },
			{XK_B, 			KEY_B },
			{XK_N, 			KEY_N },
			{XK_comma, 		KEY_M },
			{XK_semicolon, 		KEY_COMMA },
			{XK_colon, 		KEY_STOP },
			{XK_exclam, 		KEY_SLASH },
			{XK_Shift_R, 		KEY_RSHIFT },
			{XK_KP_Multiply, 	KEY_ASTERISK },
			{XK_Alt_L, 		KEY_ALT },
			{XK_space, 		KEY_SPACE },
			{XK_Caps_Lock, 		KEY_CAPSLOCK },
			{XK_F1, 		KEY_F1 },
			{XK_F2, 		KEY_F2 },
			{XK_F3, 		KEY_F3 },
			{XK_F4, 		KEY_F4 },
			{XK_F5, 		KEY_F5 },
			{XK_F6, 		KEY_F6 },
			{XK_F7, 		KEY_F7 },
			{XK_F8, 		KEY_F8 },
			{XK_F9, 		KEY_F9 },
			{XK_F10, 		KEY_F10 },
			{XK_Num_Lock, 		KEY_NUMLOCK },
			{XK_Scroll_Lock, 	KEY_SCRLOCK },
			{XK_KP_7, 		KEY_7_PAD },
			{XK_KP_8, 		KEY_8_PAD },
			{XK_KP_9, 		KEY_9_PAD },
			{XK_KP_Subtract, 	KEY_MINUS_PAD },
			{XK_KP_4, 		KEY_4_PAD },
			{XK_KP_5, 		KEY_5_PAD },
			{XK_KP_6, 		KEY_6_PAD },
			{XK_KP_Add, 		KEY_PLUS_PAD },
			{XK_KP_1, 		KEY_1_PAD },
			{XK_KP_2, 		KEY_2_PAD },
			{XK_KP_3, 		KEY_3_PAD },
			{XK_KP_0, 		KEY_0_PAD },
			{XK_KP_Decimal, 	KEY_DEL_PAD },
			{XK_less, 		KEY_BACKSLASH2 },
			{XK_F11, 		KEY_F11 },
			{XK_F12, 		KEY_F12 },
			{XK_KP_Enter, 		KEY_ENTER_PAD },
			{XK_Control_R, 		KEY_RCONTROL },
			{XK_KP_Divide, 		KEY_SLASH_PAD },
			{XK_Mode_switch, 	KEY_ALTGR },
			{XK_Home, 		KEY_HOME },
			{XK_Up, 		KEY_UP },
			{XK_Page_Up, 		KEY_PGUP },
			{XK_Left, 		KEY_LEFT },
			{XK_Right, 		KEY_RIGHT },
			{XK_End, 		KEY_END },
			{XK_Down, 		KEY_DOWN },
			{XK_Page_Down, 		KEY_PGDN },
			{XK_Insert, 		KEY_INSERT },
			{XK_Delete, 		KEY_DEL } };



/* SetPointer:
 *  Slectionne le pointeur actif.
 */
static void SetPointer(int pointer)
{
    if (pointer==TO8_MOUSE)
    {
	XDefineCursor(display, screen_win, null_cursor);
	installed_pointer=TO8_MOUSE;
    }
    else if (pointer==TO8_LIGHTPEN)
	installed_pointer=TO8_LIGHTPEN;
}



/* InitDisplay:
 *  Ouvre la connexion avec le serveur X et initialise le clavier.
 */
void InitDisplay(void)
{
    register int i;
    int ret1, ret2, ret3;

    /* Connexion au serveur X */
    display = GDK_DISPLAY();
    screen=DefaultScreen(display);

    /* Calcul de la table de conversion des keycodes */
    for (i=0; i<KB_SIZE; i++)
        x11_to_dos[XKeysymToKeycode(display,keyconv[i].keysym)]=keyconv[i].keycode;

    /* Test de prsence de l'extension MIT-SHM */
    mit_shm_enabled = XQueryExtension(display, "MIT-SHM", &ret1, &ret2, &ret3);
}



/* InitWindow:
 *  Met en place les proprits et cre la fentre principale,
 *  calcule la position et cre la fentre de l'cran.
 */
void InitWindow(int argc, char *argv[], int x, int y, int user_flags)
{
    char *window_name="Teo - l'mulateur TO8";
    XSizeHints *size_hints;
    XWMHints *wm_hints;
    XClassHint *class_hints;
    XTextProperty windowName;
    Window window_win;
    XSetWindowAttributes win_attrb;
    Pixmap null_bitmap;
    XColor color;

    printf("Cration de la fentre principale...");

    if ( !(size_hints=XAllocSizeHints()) || !(wm_hints = XAllocWMHints()) ||
	                                  !(class_hints = XAllocClassHint()) )
    {
        fprintf(stderr,"erreur d'allocation mmoire X\n");
        exit(EXIT_FAILURE);
    }

    /* Cration de la fentre principale */
    screen_win=XCreateSimpleWindow(display, RootWindow(display,screen), x, y,
      TO8_SCREEN_W*2, TO8_SCREEN_H*2, 4, WhitePixel(display, screen),
        BlackPixel(display,screen));

    XSelectInput(display, screen_win, ExposureMask | FocusChangeMask | KeyPressMask | KeyReleaseMask);

    /* Cration du curseur invisible */
    null_bitmap = XCreateBitmapFromData(display, screen_win, "", 1, 1);
    null_cursor = XCreatePixmapCursor(display, null_bitmap, null_bitmap, &color, &color, 0, 0);
    XDefineCursor(display, screen_win, null_cursor);
    XFreePixmap(display, null_bitmap);
    to8_SetPointer=SetPointer;

    /* Cration de la fentre de l'cran */
    win_attrb.win_gravity=CenterGravity;
    win_attrb.event_mask=ButtonPressMask | ButtonReleaseMask | PointerMotionMask;
    window_win=XCreateWindow(display, screen_win, TO8_BORDER_W*2, TO8_BORDER_H*2, TO8_WINDOW_W*2, TO8_WINDOW_H*2, 0, CopyFromParent, InputOnly, CopyFromParent, CWWinGravity|CWEventMask, &win_attrb);

    /* Dimensionnement de la fentre */
    size_hints->min_width=TO8_SCREEN_W*2;
    size_hints->min_height=TO8_SCREEN_H*2;
    size_hints->max_width=TO8_SCREEN_W*2;
    size_hints->max_height=TO8_SCREEN_H*2;
    size_hints->flags=PPosition | PSize | PMinSize | PMaxSize | user_flags;

    /* Formatage du nom de la fentre */
    if (!XStringListToTextProperty(&window_name, 1, &windowName))
    {
        fprintf(stderr,"erreur d'allocation de structure X\n");
        exit(EXIT_FAILURE);
    }

    /* Renseignement du Window Manager */
    wm_hints->initial_state = NormalState;
    wm_hints->input = True;
    wm_hints->flags = StateHint | InputHint;

    class_hints->res_name=PROG_NAME;
    class_hints->res_class=PROG_CLASS;

    XSetWMProperties(display, screen_win, &windowName, NULL, argv, argc, size_hints, wm_hints, class_hints);

    XMapWindow(display, window_win);
    XMapWindow(display, screen_win);

    printf("ok\n");
}



/* HandleEvents:
 *  Lit et traite les vnements envoys par le serveur X.
 */
void HandleEvents(void)
{
    XEvent ev;
    static int need_modifiers_reset;
           int key;

    while (XPending(display))
    {
        XNextEvent(display, &ev);

        switch (ev.type)
        {
	    case FocusIn:  /* screen_win */
                to8_InputReset(0, 0);
		need_modifiers_reset=True;
		break;

	    case KeyRelease:  /* screen_win */
                to8_HandleKeyPress(x11_to_dos[ev.xkey.keycode], True);
                break;
              
	    case KeyPress:  /* screen_win */
		if (need_modifiers_reset)
		{
		    int value = 0;

		    if (ev.xkey.state&ShiftMask)
			value |= TO8_SHIFT_FLAG;

		    if (ev.xkey.state&ControlMask)
			value |= TO8_CTRL_FLAG;

		    if (ev.xkey.state&(1<<13))  /* should be Mod3Mask */
			value |= TO8_ALTGR_FLAG;

		    if (ev.xkey.state&Mod2Mask)
			value |= TO8_NUMLOCK_FLAG;

		    if (ev.xkey.state&LockMask)
			value |= TO8_CAPSLOCK_FLAG;

		    to8_InputReset((1<<TO8_MAX_FLAG)-1, value); 
		    need_modifiers_reset=False;
		}

                key=x11_to_dos[ev.xkey.keycode];

                if (key==KEY_ESC)
                    teo.command=CONTROL_PANEL;
                else    
                    to8_HandleKeyPress(key, False);

                break;

	    case Expose:  /* screen_win */
		RetraceScreen(ev.xexpose.x, ev.xexpose.y, ev.xexpose.width, ev.xexpose.height);
#ifdef DEBUG
		fprintf(stderr, "Expose event: x=%d, y=%d, w=%d, h=%d\n", ev.xexpose.x, ev.xexpose.y, ev.xexpose.width, ev.xexpose.height);
#endif     
                break;
                 
	    case ButtonRelease:  /* window_win */
                if (ev.xbutton.button==Button1)
                    to8_HandleMouseClick(1, True);
                else if (ev.xbutton.button==Button3)
                    to8_HandleMouseClick(2, True);
                break;
                                             
            case ButtonPress:  /* window_win */
                if (ev.xbutton.button==Button1)
                    to8_HandleMouseClick(1, False);
                else if (ev.xbutton.button==Button3)
		{
		    static int pointer_on;

		    if (installed_pointer==TO8_MOUSE)
			to8_HandleMouseClick(2, False);
		    else 
		    {
			pointer_on^=1;
			XDefineCursor(display,screen_win, pointer_on ? None : null_cursor);
		    }
		}
                break;

            case MotionNotify:  /* window_win */
                to8_HandleMouseMotion(ev.xbutton.x/2, ev.xbutton.y/2);
                break;

        } /* end of switch(ev.type) */
    }  /* end of while XPending */
}
