/*
 *    TTTTTTTTTTTTTT  EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *    TTTTTTTTTTTTTT  EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EEEEEEEEEE      OO          OO
 *          TT        EEEEEEEEEE      OO          OO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *          TT        EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *
 *                  L'mulateur Thomson TO8
 *
 *  Copyright (C) 1997-2003 Gilles Ftis, Eric Botcazou, Alexandre Pukall,
 *                          Jrmie Guillaume
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : linux/main.c
 *  Version    : 1.7.5
 *  Cr par   : Eric Botcazou octobre 1999
 *  Modifi par: Eric Botcazou 04/11/2003
 *
 *  Boucle principale de l'mulateur.
 */


#ifndef SCAN_DEPEND
   #include <locale.h>
   #include <stdio.h>
   #include <stdlib.h>
   #include <string.h>
   #include <signal.h>
   #include <unistd.h>
   #include <sys/time.h>
   #include <gtk/gtk.h>
   #include <X11/Xlib.h>
   #include <X11/Xresource.h>
   #include <X11/Xutil.h>
#endif

#include "linux/disk.h"
#include "linux/display.h"
#include "linux/graphic.h"
#include "linux/gui.h"
#include "linux/main.h"
#include "linux/sound.h"
#include "to8.h"


struct EmuTO teo={
    True,
    True,
    NONE
};

static int frame;           /* compteur de frame vido */
static volatile int tick;   /* compteur du timer */

#define OP_TABLE_SIZE  9
static XrmOptionDescRec op_table[OP_TABLE_SIZE]={
{"-help",	".help",		XrmoptionIsArg,		NULL},
{"-m",		".memo7",		XrmoptionSepArg,	NULL},
{"-fast",	".speed",		XrmoptionNoArg,		"fast"},
{"-nosound",	".sound",		XrmoptionNoArg,		"off"},
{"-geometry",	".geometry",		XrmoptionSepArg,	NULL},
{"-noshm",      ".shm",                 XrmoptionNoArg,         "off"},
{"-display",	".display",		XrmoptionSepArg,	NULL},
{"--enable-direct-write", ".direct",    XrmoptionNoArg,         "on"},
{"-xrm",	NULL,			XrmoptionResArg,	NULL}
};


static void Timer(int sigtype)
{
    tick++;
    signal(SIGALRM,Timer);
    (void) sigtype;
}



/* RunTO8:
*  Boucle principale de l'mulateur.
 */
static void RunTO8(void)
{
    const struct itimerval timer_value={ {0,1e6/TO8_FRAME_FREQ},
               				 {0,1e6/TO8_FRAME_FREQ} };

    if (!teo.sound_enabled)
    {
	signal(SIGALRM,Timer);
	setitimer(ITIMER_REAL, &timer_value, NULL);
    }

    frame=1;
    to8_ColdReset();

    do   /* boucle principale de l'mulateur */
    {
        teo.command=NONE;
	tick=frame;

        do  /* boucle d'mulation */
        {
            to8_DoFrame();

	    RefreshScreen();

            HandleEvents();

            if (teo.exact_speed)  /* synchronisation sur frquence relle */
            {
                if (teo.sound_enabled)
		    PlaySoundBuffer();
                else if (frame==tick)
	   	    pause();  /* on attend le SIGALARM du timer */
            }

            frame++;
        }
	while (teo.command==NONE);  /* fin de la boucle d'mulation */

	if (teo.command==CONTROL_PANEL)
	    ControlPanel();

	if (teo.command==RESET)
	    to8_Reset();

	if (teo.command==COLD_RESET)
	    to8_ColdReset();

    }
    while (teo.command != QUIT);  /* fin de la boucle principale */
}



/* GetUserInput:
 *  Lit toutes les options spcifies par l'utilisateur dans
 *  l'ordre de priorit dcroissante suivant:
 *    - ligne de commande,
 *    - ressource de l'cran mise en place par xrdb,
 *    - fichier .Xdefaults dans le rpertoire utilisateur,
 *  et les fusionne dans la base de donnes user_db.
 */
static XrmDatabase GetUserInput(int *argc, char *argv[])
{
    char file_name[FILENAME_LENGTH];
    char *str_type[20];
    XrmDatabase commandline_db=NULL, user_db;
    XrmValue value;

    XrmParseCommand(&commandline_db,op_table,OP_TABLE_SIZE,PROG_NAME,argc,argv);

    /* Demande d'aide ? */
    if (XrmGetResource(commandline_db,PROG_NAME".help",PROG_CLASS".Help",str_type, &value))
    {
        printf("Usage:\n");
        printf("           %s [options...]\n",argv[0]);
        printf("o les options sont prises parmi les suivantes:\n");
        printf("    -help                   affiche cette aide\n");
        printf("    -m file.m7	     	    place la cartouche file.m7 dans le lecteur\n");
        printf("    -fast                   active la pleine vitesse de l'mulateur\n");
        printf("    -nosound                supprime le son de l'mulateur\n");
        printf("    -geometry +x+y          spcifie la position de la fentre\n");
	printf("    -noshm                  dsactive l'utilisation de l'extension MIT-SHM\n");
        printf("    -display displayname    spcifie le serveur X  utiliser\n");
        exit(EXIT_SUCCESS);
    }

    /* On a besoin de display pour accder  la configuration utilisateur */
    InitDisplay();

    if (XScreenResourceString(DefaultScreenOfDisplay(display)) != NULL)
        user_db=XrmGetStringDatabase(XScreenResourceString(DefaultScreenOfDisplay(display)));
    else if (XResourceManagerString(display) != NULL)
        user_db=XrmGetStringDatabase(XResourceManagerString(display));
    else
    {
        strcat(strcpy(file_name,getenv("HOME")),"./Xdefaults");
        user_db=XrmGetFileDatabase(file_name);
    }

    XrmMergeDatabases(commandline_db, &user_db);

    return user_db;
}



/* SetParameters:
 *  Fixe les paramtres de l'mulation,  partir des options de
 *  l'utilisateur ou par dfaut.
 */
static void SetParameters(char memo_name[], int *x, int *y, int *user_flags, int *direct_write_support, XrmDatabase user_db)
{
    char *str_type[20], buffer[20];
    int flags;
    XrmValue value;

    /* Fichier qui dcrit la cartouche place dans le lecteur */
    if (XrmGetResource(user_db,PROG_NAME".memo7",PROG_CLASS".Memo7", str_type, &value))
        strncpy(memo_name, value.addr, value.size);

    /* Vitesse de l'mulation: relle (celle du TO8) ou rapide (celle du PC) */
    if (XrmGetResource(user_db,PROG_NAME".speed",PROG_CLASS".Speed", str_type, &value))
    {
        if (!strncmp(value.addr,"fast",value.size))
            teo.exact_speed=False;
        else if (strncmp(value.addr,"true",value.size))
        {
            fprintf(stderr,"%s: spcification de vitesse invalide (voir %s -help)\n", PROG_NAME, PROG_NAME);
            exit(EXIT_FAILURE);
        }
    }

    /* Activation de l'mulation sonore */
    if (XrmGetResource(user_db,PROG_NAME".sound",PROG_CLASS".Sound", str_type, &value))
    {
        if (!strncmp(value.addr,"off",value.size))
            teo.sound_enabled=False;
        else if (strncmp(value.addr,"on",value.size))
        {
            fprintf(stderr,"%s: spcification de l'mulation sonore invalide (voir %s -help)\n", PROG_NAME, PROG_NAME);
            exit(EXIT_FAILURE);
        }
    }

    /* Gomtrie de la fentre */
    if (XrmGetResource(user_db,PROG_NAME".geometry",PROG_CLASS".Geometry",str_type, &value))
    {
	int w,h;

        strncpy(buffer, value.addr, value.size);
        flags=XParseGeometry(buffer, x, y, &w, &h);

	/* position x */
        if (flags&XValue)
        {
            *user_flags|=USPosition;

            if (flags&XNegative)
                *x=DisplayWidth(display,screen)+*x-TO8_SCREEN_W*2;
        }

	/* position y */
        if (flags&YValue)
        {
            *user_flags|=USPosition;

            if (flags&YNegative)
                *y=DisplayHeight(display,screen)+*y-TO8_SCREEN_H*2;
        }
    }

    /* Activation de l'extension MIT-SHM */
    if (XrmGetResource(user_db,PROG_NAME".shm",PROG_CLASS".Shm",str_type, &value))
    {
        if (!strncmp(value.addr,"off",value.size))
            mit_shm_enabled=False;
        else if (strncmp(value.addr,"on",value.size))
        {
            fprintf(stderr,"%s: spcification de l'extension MIT-SHM invalide\n",PROG_NAME);
            exit(EXIT_FAILURE);
        }
    }

    /* Autorisation d'criture directe */
    if (XrmGetResource(user_db,PROG_NAME".direct",PROG_CLASS".direct",str_type, &value))
    {
        if (!strncmp(value.addr,"on",value.size))
            *direct_write_support = True;
    }
}



/* ExitMessage:
 *  Affiche un message de sortie et sort du programme.
 */
static void ExitMessage(const char msg[])
{
    fprintf(stderr, "%s\n", msg);
    exit(EXIT_FAILURE);
}


#define IS_3_INCHES(drive) ((drive_type[drive]>2) && (drive_type[drive]<7))

/* main:
 *  Point d'entre du programme appel par Linux.
 */
int main(int argc, char *argv[])
{
    int i, x=0, y=0, user_flags=0;
    char version_name[]="Teo "TO8_VERSION_STR" (Linux/X11)";
    char memo_name[FILENAME_LENGTH]="\0";
    int direct_support = 0, direct_write_support = FALSE;
    int drive_type[4];
    XrmDatabase user_db;

    /* Initialisation du serveur X */
    setlocale(LC_ALL, "fr_FR");
    gtk_init(&argc, &argv);

    /* Initialisation du module de gestion des ressources */
    XrmInitialize();

    /* Mise en commun dans user_db toutes les commandes et options spcifies
       par l'utilisateur */
    user_db = GetUserInput(&argc, argv);

    /* Mise en place des paramtres de l'mulation */
    SetParameters(memo_name, &x, &y, &user_flags, &direct_write_support, user_db);
    XrmDestroyDatabase(user_db);

    /* Affichage du message de bienvenue du programme */
    printf("Voici %s l'mulateur Thomson TO8.\n",version_name);
    printf("Copyright (C) 1997-2003 Gilles Ftis, Eric Botcazou, Alexandre Pukall.\n\n");
    printf("Touches: [ESC] Panneau de contrle\n\n");

    /* Initialisation du TO8 */
    printf("Initialisation de l'mulateur...");
    fflush(stdout);

    if ( to8_Init(TO8_NJOYSTICKS) == TO8_ERROR )
        ExitMessage(to8_error_msg);

    printf("ok\n");

    /* Chargement de la cartouche */
    if (memo_name[0])
        to8_LoadMemo7(memo_name);

    /* Initialisation de l'interface d'accs direct */
    InitDirectDisk(drive_type, direct_write_support);

    /* Dtection des lecteurs supports (3"5 seulement) */
    for (i=0; i<4; i++)
    {
         if (IS_3_INCHES(i))
             direct_support |= (1<<i);
    }

    /* Cration de la fentre principale de l'mulateur */
    InitWindow(argc, argv, x, y, user_flags);

     /* Initialisation de l'interface utilisateur */
    InitGUI(version_name, direct_support);

    /* Initialisation des modules graphique, sonore et disquette */
    InitGraphic();
    InitSound();

    /* Et c'est parti !!! */
    printf("Lancement de l'mulation...\n");
    RunTO8();

    /* Mise au repos de l'interface d'accs direct */
    ExitDirectDisk();

    /* Sortie de l'mulateur */
    printf("\nA bientt !\n");
    exit(EXIT_SUCCESS);
}
