/*
 *    TTTTTTTTTTTTTT  EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *    TTTTTTTTTTTTTT  EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EEEEEEEEEE      OO          OO
 *          TT        EEEEEEEEEE      OO          OO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *          TT        EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *
 *                  L'mulateur Thomson TO8
 *
 *  Copyright (C) 1997-2003 Gilles Ftis, Eric Botcazou, Alexandre Pukall,
 *                          Jrmie Guillaume
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : win/gui.c
 *  Version    : 1.7.5
 *  Cr par   : Eric Botcazou 28/11/2000
 *  Modifi par: Eric Botcazou 28/10/2003
 *
 *  Interface utilisateur Windows native.
 */


#ifndef SCAN_DEPEND
   #include <stdlib.h>
   #include <string.h>
   #include <windows.h>
   #include <commctrl.h>
#endif

#include "alleg/gfxdrv.h"
#include "alleg/main.h"
#include "alleg/sound.h"
#include "win/dialog.rh"
#include "to8.h"


/* Nom du programme pour les botes de dialogue. */
#define PROGNAME_STR  "Teo"


/* ressources globales de l'application */
HINSTANCE prog_inst;
HWND prog_win;
HICON prog_icon;


/* le support des listes associes aux comboboxes de l'interface est assur par une
   implmentation rduite en Standard C du container vector du Standard C++ */

#define CHUNK_SIZE   5
#define NOT_FOUND   -1
#define BUFFER_SIZE  512

/* pair<string, string> */
struct STRING_PAIR {
    char *fullname;
    char *label;
};

/* vector< pair<string, string> > */
struct FILE_VECTOR {
    int id;
    int size;
    int capacity;
    int selected;
    int protection;
    struct STRING_PAIR *file;   
};



/* vector< pair<string, string> >::push_back( make_pair(string(), string()) ):
 *  Ajoute une paire de chanes au vecteur spcifi en l'agrandissant si ncessaire.
 */
static int push_back(struct FILE_VECTOR *vector, const char fullname[], const char label[])
{
   if (vector->size == vector->capacity)
   {
       /* on alloue un espace mmoire plus important */
       struct STRING_PAIR *temp;
       vector->capacity += CHUNK_SIZE;
       temp = malloc( vector->capacity*sizeof(struct STRING_PAIR) );
       memcpy(temp, vector->file, vector->size*sizeof(struct STRING_PAIR));
       free(vector->file);
       vector->file = temp;
   }

   vector->file[vector->size].fullname = malloc( (strlen(fullname)+1)*sizeof(char) );
   strcpy(vector->file[vector->size].fullname, fullname);

   vector->file[vector->size].label = malloc( (strlen(label)+1)*sizeof(char) );
   strcpy(vector->file[vector->size].label, label);

   vector->size++;

   return (vector->size-1);
}



/* find:
 *  Cherche un nom de fichier dans un vecteur et retourne le cas chant son index.
 */
static int find(struct FILE_VECTOR *vector, const char fullname[])
{
   int i;

   for (i=0; i<vector->size; i++)
      if (!strcmp(vector->file[i].fullname, fullname))
         return i;

   return NOT_FOUND;
}



/* extract_dir:
 *  Extrait le nom du rpertoire du nom complet du fichier spcifi.
 */
static void extract_dir(char dir[], const char fullname[])
{
   int len = strlen(fullname);

   strcpy(dir, fullname);

   while (--len > 0)
      if (dir[len] == '\\')
      {
         dir[len] = '\0';
         break;
      }
}
 


/* get_filename:
 *  Retourne le nom du fichier  partir du nom complet du fichier spcifi.
 */
static const char* get_filename(const char fullname[])
{
   int len = strlen(fullname);

   while (--len > 0)
      if (fullname[len] == '\\')
         return fullname + len + 1;

   return fullname;
}



/***********************************************/
/* mthodes du vecteur associ  la cartouche  */
/***********************************************/


static void init_file_memo(struct FILE_VECTOR *vector, HWND hDlg)
{
   static int first = 1;
   const char *str_p;
   int i;

   if (first)
   {
      str_p = to8_GetMemo7Filename();
      if (*str_p)
         push_back(vector, str_p, to8_GetMemo7Label());

      first = 0;
   }

   if (vector->size)
   {
      for (i=0; i<vector->size; i++)
         SendDlgItemMessage(hDlg, MEMO7_COMBO, CB_ADDSTRING, 0, (LPARAM) vector->file[i].label);
   }
   else
      SendDlgItemMessage(hDlg, MEMO7_COMBO, CB_ADDSTRING, 0, (LPARAM) "aucune cartouche");

   SendDlgItemMessage(hDlg, MEMO7_COMBO, CB_SETCURSEL, vector->selected, 0);
}


static void open_file_memo(struct FILE_VECTOR *vector, HWND hDlg)
{
   OPENFILENAME openfilename;
   char buffer[BUFFER_SIZE], dir[BUFFER_SIZE];
   int index;

   /* initialisation du slecteur de fichiers */
   memset(&openfilename, 0, sizeof(OPENFILENAME));

   openfilename.lStructSize = sizeof(OPENFILENAME);
   openfilename.hwndOwner = hDlg;
   openfilename.lpstrFilter = "Fichiers Mmo7\0*.m7\0";
   openfilename.nFilterIndex = 1;
   openfilename.lpstrFile = buffer;
   openfilename.nMaxFile = BUFFER_SIZE;

   if (vector->size)
   {
      extract_dir(dir, vector->file[vector->selected].fullname);
      openfilename.lpstrInitialDir = dir;
   }
   else
      openfilename.lpstrInitialDir = ".\\memo7";

   openfilename.lpstrTitle = "Choisissez votre cartouche:";
   openfilename.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_NOCHANGEDIR;
   openfilename.lpstrDefExt ="m7";

   buffer[0] = '\0';

   if (GetOpenFileName(&openfilename))
   {
      if (to8_LoadMemo7(openfilename.lpstrFile))
      {        
         index = find(vector, openfilename.lpstrFile);
                         
         if (index == NOT_FOUND)
         {
            index = push_back(vector, openfilename.lpstrFile, to8_GetMemo7Label());

            if (!index)
               SendDlgItemMessage(hDlg, MEMO7_COMBO, CB_DELETESTRING, 0, 0);
                             
            SendDlgItemMessage(hDlg, MEMO7_COMBO, CB_ADDSTRING, 0, (LPARAM) vector->file[index].label);
         }

         SendDlgItemMessage(hDlg, MEMO7_COMBO, CB_SETCURSEL, index, 0);
         vector->selected = index;
         teo.command = COLD_RESET;
      }
      else
         MessageBox(hDlg, to8_error_msg, PROGNAME_STR, MB_OK | MB_ICONERROR);
   }
}


static void select_file_memo(struct FILE_VECTOR *vector, HWND hDlg)
{
   int index;

   if (vector->size)
   {
      index = SendDlgItemMessage(hDlg, MEMO7_COMBO, CB_GETCURSEL, 0, 0);

      if (to8_LoadMemo7(vector->file[index].fullname))
      {
         vector->selected= index;
         teo.command = COLD_RESET;
      }
      else
      {
         MessageBox(hDlg, to8_error_msg, PROGNAME_STR, MB_OK | MB_ICONERROR);
         SendDlgItemMessage(hDlg, MEMO7_COMBO, CB_SETCURSEL, vector->selected, 0);
      }
   }
}



/***********************************************/
/* mthodes du vecteur associ  la cassette   */
/***********************************************/


static void init_file_cass(struct FILE_VECTOR *vector, HWND hDlg)
{
   static int first = 1;
   const char *str_p;
   int i;

   if (first)
   {
      str_p = to8_GetK7Filename();
      if (*str_p)
         push_back(vector, str_p, get_filename(str_p));

      first = 0;
   }

   if (vector->size)
   {
      for (i=0; i<vector->size; i++)
         SendDlgItemMessage(hDlg, K7_COMBO, CB_ADDSTRING, 0, (LPARAM) vector->file[i].label);
   }
   else
      SendDlgItemMessage(hDlg, K7_COMBO, CB_ADDSTRING, 0, (LPARAM) "aucune cassette");

   SendDlgItemMessage(hDlg, K7_COMBO, CB_SETCURSEL, vector->selected, 0);
}


static void open_file_cass(struct FILE_VECTOR *vector, HWND hDlg)
{
   OPENFILENAME openfilename;
   char buffer[BUFFER_SIZE], dir[BUFFER_SIZE];
   int ret, index;

   /* initialisation du slecteur de fichiers */
   memset(&openfilename, 0, sizeof(OPENFILENAME));

   openfilename.lStructSize = sizeof(OPENFILENAME);
   openfilename.hwndOwner = hDlg;
   openfilename.lpstrFilter = "Fichiers K7\0*.k7\0";
   openfilename.nFilterIndex = 1;
   openfilename.lpstrFile = buffer;
   openfilename.nMaxFile = BUFFER_SIZE;

   if (vector->size)
   {
      extract_dir(dir, vector->file[vector->selected].fullname);
      openfilename.lpstrInitialDir = dir;
   }
   else
      openfilename.lpstrInitialDir = ".\\k7";

   openfilename.lpstrTitle = "Choisissez votre cassette:";
   openfilename.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_NOCHANGEDIR;
   openfilename.lpstrDefExt ="k7";

   buffer[0] = '\0';

   if (GetOpenFileName(&openfilename))
   {
      ret = to8_LoadK7(openfilename.lpstrFile);

      if (ret == TO8_ERROR)
         MessageBox(hDlg, to8_error_msg, PROGNAME_STR, MB_OK | MB_ICONERROR);
      else
      {        
         index = find(vector, openfilename.lpstrFile);

         if (index == NOT_FOUND)
         {
            index = push_back(vector, openfilename.lpstrFile, openfilename.lpstrFile + openfilename.nFileOffset);

            if (!index)
               SendDlgItemMessage(hDlg, K7_COMBO, CB_DELETESTRING, 0, 0);
                             
            SendDlgItemMessage(hDlg, K7_COMBO, CB_ADDSTRING, 0, (LPARAM) vector->file[index].label);
         }

         SendDlgItemMessage(hDlg, K7_COMBO, CB_SETCURSEL, index, 0);
         vector->selected = index;

         if ((ret == TO8_READ_ONLY) && !vector->protection)
         {
            MessageBox(hDlg, "Attention: criture impossible.", PROGNAME_STR, MB_OK | MB_ICONINFORMATION);
            CheckDlgButton(hDlg, K7_PROT_CHECK, BST_CHECKED); 
            vector->protection = TRUE;
         }
      }  
   }
}


static void select_file_cass(struct FILE_VECTOR *vector, HWND hDlg)
{
   int ret, index;

   if (vector->size)
   {
      index = SendDlgItemMessage(hDlg, K7_COMBO, CB_GETCURSEL, 0, 0);
      ret = to8_LoadK7(vector->file[index].fullname);

      if (ret == TO8_ERROR)
      {
         MessageBox(hDlg, to8_error_msg, PROGNAME_STR, MB_OK | MB_ICONERROR);
         SendDlgItemMessage(hDlg, K7_COMBO, CB_SETCURSEL, vector->selected, 0);
      }
      else
      {
         vector->selected = index;
           
         if ((ret == TO8_READ_ONLY) && !vector->protection)
         {
            MessageBox(hDlg, "Attention: criture impossible.", PROGNAME_STR, MB_OK | MB_ICONINFORMATION);
            CheckDlgButton(hDlg, K7_PROT_CHECK, BST_CHECKED); 
            vector->protection = TRUE;
         }
      }
   }
}


static void init_check_cass(struct FILE_VECTOR *vector, HWND hDlg)
{
   if (!vector->size)
      vector->protection = TRUE;

   CheckDlgButton(hDlg, K7_PROT_CHECK, vector->protection ? BST_CHECKED : BST_UNCHECKED);
}


static void change_protection_cass(struct FILE_VECTOR *vector, HWND hDlg)
{
   if (IsDlgButtonChecked(hDlg, K7_PROT_CHECK))
   {
      to8_SetK7Mode(TO8_READ_ONLY);
      vector->protection = TRUE;
   }
   else
   {
      if (to8_SetK7Mode(TO8_READ_WRITE) == TO8_READ_ONLY)
      {
         MessageBox(hDlg, "Ecriture impossible sur ce support.", PROGNAME_STR, MB_OK | MB_ICONINFORMATION);
         CheckDlgButton(hDlg, K7_PROT_CHECK, BST_CHECKED); 
         vector->protection = TRUE;
      }
      else
         vector->protection = FALSE;
   }
}

#define COUNTER_MAX  999

static void update_counter_cass(HWND hDlg)
{
   int counter = to8_GetK7Counter();
   SetDlgItemInt(hDlg, K7_COUNTER_EDIT, counter, FALSE);
   SendDlgItemMessage(hDlg, K7_UPDOWN, UDM_SETPOS, 0, MAKELONG(counter, 0));
}



/***********************************************/
/* mthodes du vecteur associ  la disquette  */
/***********************************************/


static void init_file_disk(struct FILE_VECTOR *vector, HWND hDlg)
{
   static int first = 1;
   const char *str_p;
   int i;

   if (first)
   {
      str_p = to8_GetDiskFilename(vector->id);
      if (*str_p)
         vector->selected = push_back(vector, str_p, get_filename(str_p));

      first = 0;
   }

   if (vector->size)
   {
      for (i=0; i<vector->size; i++)
         SendDlgItemMessage(hDlg, DISK0_COMBO+vector->id, CB_ADDSTRING, 0, (LPARAM) vector->file[i].label);
   }
   else
      SendDlgItemMessage(hDlg, DISK0_COMBO+vector->id, CB_ADDSTRING, 0, (LPARAM) "aucune disquette");

   SendDlgItemMessage(hDlg, DISK0_COMBO+vector->id, CB_SETCURSEL, vector->selected, 0);
}


static void open_file_disk(struct FILE_VECTOR *vector, HWND hDlg)
{
   OPENFILENAME openfilename;
   char buffer[BUFFER_SIZE], dir[BUFFER_SIZE];
   int ret, index;

   /* initialisation du slecteur de fichiers */
   memset(&openfilename, 0, sizeof(OPENFILENAME));

   openfilename.lStructSize = sizeof(OPENFILENAME);
   openfilename.hwndOwner = hDlg;
   openfilename.lpstrFilter = "Fichiers SAP\0*.sap\0";
   openfilename.nFilterIndex = 1;
   openfilename.lpstrFile = buffer;
   openfilename.nMaxFile = BUFFER_SIZE;

   if (vector->size)
   {
      extract_dir(dir, vector->file[vector->selected].fullname);
      openfilename.lpstrInitialDir = dir;
   }
   else
      openfilename.lpstrInitialDir = ".\\disks";

   openfilename.lpstrTitle = "Choisissez votre disquette:";
   openfilename.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_NOCHANGEDIR;
   openfilename.lpstrDefExt ="sap";

   buffer[0] = '\0';

   if (GetOpenFileName(&openfilename))
   {
      ret = to8_LoadDisk(vector->id, openfilename.lpstrFile);

      if (ret == TO8_ERROR)
         MessageBox(hDlg, to8_error_msg, PROGNAME_STR, MB_OK | MB_ICONERROR);
      else
      {
         index = find(vector, openfilename.lpstrFile);

         if (index == NOT_FOUND)
         {
            index = push_back(vector, openfilename.lpstrFile, openfilename.lpstrFile + openfilename.nFileOffset);

            if (!index)
               SendDlgItemMessage(hDlg, DISK0_COMBO+vector->id, CB_DELETESTRING, 0, 0);

            SendDlgItemMessage(hDlg, DISK0_COMBO+vector->id, CB_ADDSTRING, 0, (LPARAM) vector->file[index].label);
         }

         SendDlgItemMessage(hDlg, DISK0_COMBO+vector->id, CB_SETCURSEL, index, 0);
         vector->selected = index;

         if ((ret == TO8_READ_ONLY) && !vector->protection)
         {
            MessageBox(hDlg, "Attention: criture impossible.", PROGNAME_STR, MB_OK | MB_ICONINFORMATION);
            CheckDlgButton(hDlg, DISK0_PROT_CHECK+vector->id, BST_CHECKED); 
            vector->protection = TRUE;
         }
      }
   }
}


static void select_file_disk(struct FILE_VECTOR *vector, HWND hDlg)
{
   int ret, index;

   if (vector->size)
   {
      index = SendDlgItemMessage(hDlg, DISK0_COMBO+vector->id, CB_GETCURSEL, 0, 0);
      ret = to8_LoadDisk(vector->id, vector->file[index].fullname);

      if (ret == TO8_ERROR)
      {
         MessageBox(hDlg, to8_error_msg, PROGNAME_STR, MB_OK | MB_ICONERROR);
         SendDlgItemMessage(hDlg, DISK0_COMBO+vector->id, CB_SETCURSEL, vector->selected, 0);
      }
      else
      {
         vector->selected = index;
           
         if ((ret == TO8_READ_ONLY) && !vector->protection)
         {
            MessageBox(hDlg, "Attention: criture impossible.", PROGNAME_STR, MB_OK | MB_ICONINFORMATION);
            CheckDlgButton(hDlg, DISK0_PROT_CHECK+vector->id, BST_CHECKED); 
            vector->protection = TRUE;
         }
      }
   }
}


static void init_check_disk(struct FILE_VECTOR *vector, HWND hDlg)
{
   if (!vector->size)
      vector->protection = FALSE;

   CheckDlgButton(hDlg, DISK0_PROT_CHECK+vector->id, vector->protection ? BST_CHECKED : BST_UNCHECKED);
}


static void change_protection_disk(struct FILE_VECTOR *vector, HWND hDlg)
{
   if (IsDlgButtonChecked(hDlg, DISK0_PROT_CHECK+vector->id))
   {
      to8_SetDiskMode(vector->id, TO8_READ_ONLY);
      vector->protection = TRUE;
   }
   else
   {
      if (to8_SetDiskMode(vector->id, TO8_READ_WRITE) == TO8_READ_ONLY)
      {
         MessageBox(hDlg, "Ecriture impossible sur ce support.", PROGNAME_STR, MB_OK | MB_ICONINFORMATION);
         CheckDlgButton(hDlg, DISK0_PROT_CHECK+vector->id, BST_CHECKED); 
         vector->protection = TRUE;
      }
      else
         vector->protection = FALSE;
   }
}



/***********************************************/
/* mthodes de la barre du volume              */
/***********************************************/

#define MIN_POS    0
#define MAX_POS    254
#define LINE_STEP  2
#define PAGE_STEP  16


static void init_bar(HWND volume_bar)
{
   SendMessage(volume_bar, TBM_SETRANGE, TRUE, MAKELONG(MIN_POS, MAX_POS));
   SendMessage(volume_bar, TBM_SETLINESIZE, TRUE, LINE_STEP);
   SendMessage(volume_bar, TBM_SETPAGESIZE, TRUE, PAGE_STEP);
   SendMessage(volume_bar, TBM_SETTICFREQ, PAGE_STEP, 0);

   SendMessage(volume_bar, TBM_SETPOS, TRUE,
      (teo.sound_enabled && teo.exact_speed) ? GetVolume()-1 : (MAX_POS-MIN_POS)/2);
   
   if (!teo.sound_enabled || !teo.exact_speed)
      EnableWindow(volume_bar, FALSE);
}


static void update_bar(HWND volume_bar, WPARAM wParam)
{
   int pos = GetVolume()-1;

   switch(LOWORD(wParam))
   {
      case TB_TOP:
         pos = MIN_POS;
         break;

      case TB_LINEUP:
         pos -= LINE_STEP;
         break;

      case TB_PAGEUP:
         pos -= PAGE_STEP;
         if (pos<MIN_POS)
            pos = MIN_POS;
         break;

      case TB_BOTTOM:
         pos = MAX_POS;
         break;

      case TB_LINEDOWN:
         pos += LINE_STEP;
         break;

      case TB_PAGEDOWN:
         pos += PAGE_STEP;
         if (pos>MAX_POS)
            pos = MAX_POS;
         break;           

      case TB_THUMBPOSITION:
      case TB_THUMBTRACK:
         pos = HIWORD(wParam);
         break;
   }

   SetVolume(pos+1);
}



static int get_open_image_name(HWND hDlg, char filename[BUFFER_SIZE], int must_exist)
{
   OPENFILENAME openfilename;

   /* initialisation du slecteur de fichiers */
   memset(&openfilename, 0, sizeof(OPENFILENAME));

   openfilename.lStructSize = sizeof(OPENFILENAME);
   openfilename.hwndOwner = hDlg;
   openfilename.lpstrFilter = "Fichiers Images\0*.img\0";
   openfilename.nFilterIndex = 1;
   openfilename.lpstrFile = filename;
   openfilename.nMaxFile = BUFFER_SIZE;
   openfilename.lpstrInitialDir = ".";

   if (must_exist)
      openfilename.lpstrTitle = "Choisissez votre image:";
   else
      openfilename.lpstrTitle = "Spcifiez un nom pour votre image:";

   openfilename.Flags = OFN_HIDEREADONLY | OFN_NOCHANGEDIR;
   if (must_exist)
      openfilename.Flags |= OFN_FILEMUSTEXIST;

   openfilename.lpstrDefExt = "img";

   filename[0] = '\0';

   return GetOpenFileName(&openfilename);
}
   


/* ControlDialogProc:
 *  Procdure du panneau de contrle.
 */
static BOOL CALLBACK ControlDialogProc(HWND hDlg, UINT Message, WPARAM wParam, LPARAM lParam)
{
   static HWND volume_bar;
   static HWND counter_updown;
   static struct FILE_VECTOR memo7, k7, disk[4] = { {0}, {1}, {2}, {3} };
   LPNM_UPDOWN nmupdown;
   char filename[BUFFER_SIZE];
   int i, counter;

   switch(Message)
   {
      case WM_INITDIALOG:
         /* mise en place de l'icne */
         SetClassLong(hDlg, GCL_HICON,   (LONG) prog_icon);
         SetClassLong(hDlg, GCL_HICONSM, (LONG) prog_icon);

         /* initialisation des boutons radio de la vitesse */
         CheckRadioButton(hDlg, EXACT_SPEED_BUTTON, MAX_SPEED_BUTTON, 
               (teo.exact_speed ? EXACT_SPEED_BUTTON : MAX_SPEED_BUTTON));

         /* initialisation de la barre du volume */
         volume_bar = GetDlgItem(hDlg, VOLUME_BAR);
         init_bar(volume_bar);

         /* initialisation du slecteur de cartouches */
         init_file_memo(&memo7, hDlg);

         /* initialisation du slecteur de cassettes */
         init_file_cass(&k7, hDlg);
         init_check_cass(&k7, hDlg);

         /* initialisation du compteur de cassettes */
         counter_updown = GetDlgItem(hDlg, K7_UPDOWN);
         SendMessage(counter_updown, UDM_SETRANGE, 0, MAKELONG(COUNTER_MAX, 0));
         update_counter_cass(hDlg);

         /* initialisation des quatre slecteurs de disquettes */
         for (i=0; i<4; i++)
         {
             init_file_disk(&disk[i], hDlg);
             init_check_disk(&disk[i], hDlg);
         }
         return TRUE;

      case WM_COMMAND:
         switch(LOWORD(wParam))
         {
            case IDOK:
               EndDialog(hDlg, IDOK);
               break;

            case QUIT_BUTTON:
               if (MessageBox(hDlg, "Voulez-vous vraiment quitter l'mulateur ?", "Teo - confirmation",
                        MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2 | MB_SYSTEMMODAL) == IDYES)
                  EndDialog(hDlg, IDCANCEL);
               break; 

            case RESET_BUTTON:
               teo.command = RESET;
               EndDialog(hDlg, IDOK);
               break;

            case COLDRESET_BUTTON:
               teo.command = COLD_RESET;
               EndDialog(hDlg, IDOK);
               break;

            case EXACT_SPEED_BUTTON:
               teo.exact_speed = TRUE;
               EnableWindow(volume_bar, TRUE);
               break;

            case MAX_SPEED_BUTTON:
               teo.exact_speed = FALSE;
               EnableWindow(volume_bar, FALSE);
               break;

            case LOAD_BUTTON:
               if (get_open_image_name(hDlg, filename, TRUE))
               {
                  if (to8_LoadImage(filename) == TO8_ERROR)
                     MessageBox(hDlg, to8_error_msg, PROGNAME_STR, MB_OK | MB_ICONERROR);
               }
               break;

            case SAVE_BUTTON:
               if (get_open_image_name(hDlg, filename, FALSE))
               {
                  if (to8_SaveImage(filename) == TO8_ERROR)
                     MessageBox(hDlg, to8_error_msg, PROGNAME_STR, MB_OK | MB_ICONERROR);
               }
               break;

            case MEMO7_MORE_BUTTON:
               open_file_memo(&memo7, hDlg);
               break;

            case K7_MORE_BUTTON:
               open_file_cass(&k7, hDlg);
               break; 

            case DISK0_MORE_BUTTON:
            case DISK1_MORE_BUTTON:
            case DISK2_MORE_BUTTON:
            case DISK3_MORE_BUTTON: 
               open_file_disk(&disk[LOWORD(wParam) - DISK0_MORE_BUTTON], hDlg);
               break;

            case MEMO7_COMBO:
               select_file_memo(&memo7, hDlg);
               break;

            case K7_COMBO:
               select_file_cass(&k7, hDlg);
               update_counter_cass(hDlg);
               break;

            case DISK0_COMBO:
            case DISK1_COMBO:
            case DISK2_COMBO:
            case DISK3_COMBO: 
               select_file_disk(&disk[LOWORD(wParam) - DISK0_COMBO], hDlg);
               break;

            case K7_PROT_CHECK:
               change_protection_cass(&k7, hDlg);
               update_counter_cass(hDlg);
               break;

            case DISK0_PROT_CHECK:
            case DISK1_PROT_CHECK:
            case DISK2_PROT_CHECK:
            case DISK3_PROT_CHECK: 
               change_protection_disk(&disk[LOWORD(wParam) - DISK0_PROT_CHECK], hDlg);
               break;

            case K7_COUNTER_EDIT:
               if (HIWORD(wParam) == EN_CHANGE)
               {
                  counter = GetDlgItemInt(hDlg, K7_COUNTER_EDIT, NULL, FALSE);
                  to8_SetK7Counter(counter);
                  SendMessage(counter_updown, UDM_SETPOS, 0, MAKELONG(counter, 0));
               }
               break;

            case K7_REWIND_BUTTON:
               to8_SetK7Counter(0);
               update_counter_cass(hDlg);
               break;

            case WM_DESTROY:
               EndDialog(hDlg, IDOK);
               break;
         }
         return TRUE;

      case WM_HSCROLL:
         if ((HWND)lParam == volume_bar)
         {
            if (teo.sound_enabled && teo.exact_speed)
               update_bar(volume_bar, wParam);
         }
         return TRUE;

      case WM_NOTIFY:
         nmupdown = (LPNM_UPDOWN)lParam;
         if ((HWND)nmupdown->hdr.hwndFrom == counter_updown)
         {
            counter = nmupdown->iPos;
            to8_SetK7Counter(counter);
            SetDlgItemInt(hDlg, K7_COUNTER_EDIT, counter, FALSE);
         }
         return TRUE;

      default:
         return FALSE;
   }
}



/* DisplayControlPanelWin:
 *  Affiche le panneau de contrle natif Windows.
 */
void DisplayControlPanelWin(void)
{
   static int first = 1;
   int ret;

   if (first)
   {
      /* initialise la librairie comctl32.dll */
      InitCommonControls();
      first = 0;
   }

   ret = DialogBox(prog_inst, "CONTROL_DIALOG", prog_win, ControlDialogProc);

   if (ret == IDCANCEL) 
      teo.command = QUIT;
}



/* StartDialogProc:
 *  Procdure de la bote de dialogue de dmarrage.
 */
static BOOL CALLBACK StartDialogProc(HWND hDlg, UINT Message, WPARAM wParam, LPARAM lParam)
{
   int ret;

   switch(Message)
   {
      case WM_INITDIALOG:
         /* mise en place de l'icne */
         SetClassLong(hDlg, GCL_HICON,   (LONG) prog_icon);
         SetClassLong(hDlg, GCL_HICONSM, (LONG) prog_icon);

         CheckRadioButton(hDlg, MODE40_BUTTON, WINDOWED_BUTTON, MODE40_BUTTON);
         return TRUE;

      case WM_COMMAND:
         switch(LOWORD(wParam))
         {
            case IDOK:
               if (IsDlgButtonChecked(hDlg, MODE40_BUTTON))
                  ret = GFX_MODE40;
               else if (IsDlgButtonChecked(hDlg, MODE80_BUTTON))
                  ret = GFX_MODE80;
               else if (IsDlgButtonChecked(hDlg, TRUECOLOR_BUTTON))
                  ret = GFX_TRUECOLOR;
               else
                  ret = GFX_WINDOW;

               EndDialog(hDlg, ret);
               break;

            case WM_DESTROY:
               EndDialog(hDlg, NO_GFX);
               exit(EXIT_SUCCESS);
               break;
         }
         return TRUE;

      default:
         return FALSE;
   }
}



/* SelectGraphicMode:
 *  Affiche la bote de dialogue de dmarrage et permet 
 *   l'utilisateur de choisir le mode graphique.
 */
void SelectGraphicMode(int *graphics_mode, int *ctrl_pressed)
{
   *graphics_mode = DialogBox(prog_inst, "START_DIALOG", NULL, StartDialogProc);

   if (ctrl_pressed)
      *ctrl_pressed = GetKeyState(VK_CONTROL)&0x8000;
}

