/*
 *    TTTTTTTTTTTTTT  EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *    TTTTTTTTTTTTTT  EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EEEEEEEEEE      OO          OO
 *          TT        EEEEEEEEEE      OO          OO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EE              OO          OO
 *          TT        EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *          TT        EEEEEEEEEEEEEE  OOOOOOOOOOOOOO
 *
 *                  L'mulateur Thomson TO8
 *
 *  Copyright (C) 1997-2003 Gilles Ftis, Eric Botcazou, Alexandre Pukall,
 *                          Jrmie Guillaume
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : win/main.c
 *  Version    : 1.7.5
 *  Cr par   : Eric Botcazou septembre 2000
 *  Modifi par: Eric Botcazou 24/10/2003
 *
 *  Boucle principale de l'mulateur.
 */


#ifndef SCAN_DEPEND
   #include <stdio.h>
   #include <stdlib.h>
   #include <string.h>
   #include <allegro.h>
   #include <winalleg.h>
#endif

#include "alleg/gfxdrv.h"
#include "alleg/gui.h"
#include "alleg/joyint.h"
#include "alleg/keybint.h"
#include "alleg/main.h"
#include "alleg/mouse.h"
#include "alleg/sound.h"
#include "alleg/state.h"
#include "win/gui.h"
#include "to8.h"


struct EmuTO teo={
    TRUE,
    TRUE,
    NONE
};

int frame;                  /* compteur de frame vido */
static volatile int tick;   /* compteur du timer       */

static void Timer(void)
{
    tick++;
}



/* RetraceCallback:
 *  Fonction callback de retraage de l'cran aprs
 *  restauration de l'application.
 */
static void RetraceCallback(void)
{
    acquire_screen();
    RetraceScreen(0, 0, SCREEN_W, SCREEN_H);
    release_screen();
}



/* RunTO8:
 *  Boucle principale de l'mulateur.
 */
static void RunTO8(int windowed_mode)
{
    if (!teo.sound_enabled)
        install_int_ex(Timer, BPS_TO_TIMER(TO8_FRAME_FREQ));

    frame=1;
    InstallPointer(TO8_MOUSE); /* la souris est le priphrique de pointage par dfaut */
    RetraceScreen(0, 0, SCREEN_W, SCREEN_H);

    do  /* boucle principale de l'mulateur */
    {
        teo.command=NONE;
        tick=frame;

        /* installation des handlers clavier, souris et son */ 
        InstallKeybint();
        InstallPointer(LAST_POINTER);

        if (teo.sound_enabled && teo.exact_speed)
            StartSound();

        do  /* boucle d'mulation */
        {
            to8_DoFrame();

            /* rafrachissement de la palette */
            if (need_palette_refresh)
                RefreshPalette();

            /* rafrachissement de l'cran */
            RefreshScreen();

            /* mise  jour de la position des joysticks */
            UpdateJoystick();

            /* synchronisation sur frquence relle */
            if (teo.exact_speed)
            {
                if (teo.sound_enabled)
                    PlaySoundBuffer();
                else
                    while (frame==tick)
	   	        Sleep(0);
            }

            frame++;
        }
        while (teo.command==NONE);  /* fin de la boucle d'mulation */

        /* dsinstallation des handlers clavier, souris et son */
        if (teo.sound_enabled && teo.exact_speed)
            StopSound();

        ShutDownPointer();
        ShutDownKeybint();

        /* xcution des commandes */
        if (teo.command==CONTROL_PANEL)
        {
            if (windowed_mode)
                DisplayControlPanelWin();
            else
                ControlPanel();
        }

        if (teo.command==SCREENSHOT)
            Screenshot();

        if (teo.command==RESET)
            to8_Reset();

        if (teo.command==COLD_RESET)
        {
            to8_ColdReset();
            InstallPointer(TO8_MOUSE);
        }
    }
    while (teo.command != QUIT);  /* fin de la boucle principale */

    if (!teo.sound_enabled)
        remove_int(Timer);
}



/* ExitMessage:
 *  Affiche un message de sortie et sort du programme.
 */
static void ExitMessage(const char msg[])
{
    allegro_exit(); /* pour viter une fentre DirectX zombie */
    MessageBox(NULL, msg, "Teo - Erreur", MB_OK | MB_ICONERROR);
    exit(EXIT_FAILURE);
}



/* WinMain:
 *  Point d'entre du programme appel par l'API Win32.
 */
int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmdLine, int nCmdShow)
{
    register int i;
    char version_name[]="Teo "TO8_VERSION_STR" (Windows/DirectX)";
    char memo_name[FILENAME_LENGTH]="\0";
    int alleg_depth, argc=0;
    char *argv[16];
    int gfx_mode=NO_GFX;
    int windowed_mode=FALSE;
    int load_state = FALSE;
    int njoy = 0;

    /* conversion de la ligne de commande Windows */
    prog_inst = hInst;
    prog_icon = LoadIcon(hInst, "thomson_ico");

    if (*lpCmdLine)
    {
        argv[argc++]=lpCmdLine++;

        while (*lpCmdLine)
            if (*lpCmdLine == ' ')
            {
                *lpCmdLine++ = '\0';
                argv[argc++]=lpCmdLine++;
            }
            else
                lpCmdLine++;
    }
    
    /* traitement des paramtres */
    for (i=0; i<argc; i++)
    {
        if (!strcmp(argv[i],"-m"))
	    strcpy(memo_name, argv[++i]);
        else if (!strcmp(argv[i],"-fast"))
            teo.exact_speed=FALSE;
        else if (!strcmp(argv[i],"-nosound"))
            teo.sound_enabled=FALSE;
        else if (!strcmp(argv[i],"-nojoy"))
            njoy = -1;
        else if (!strcmp(argv[i],"-mode40"))
            gfx_mode=GFX_MODE40;
        else if (!strcmp(argv[i],"-mode80"))
            gfx_mode=GFX_MODE80;
        else if (!strcmp(argv[i],"-truecolor"))
            gfx_mode=GFX_TRUECOLOR;
        else if (!strcmp(argv[i],"-window"))
            gfx_mode=GFX_WINDOW;
        else if (!strcmp(argv[i],"-loadstate"))
            load_state = TRUE;
        else
            ExitMessage("Paramtre inconnu.");
    }

    /* slection du mode graphique */
    if (gfx_mode == NO_GFX) {
        if (load_state)
            SelectGraphicMode(&gfx_mode, NULL);
        else
            SelectGraphicMode(&gfx_mode, &load_state);
    }

    /* l'initialisation de l'interface clavier, qui utilise un appel GDI, doit avoir lieu
       avant celle du module clavier d'Allegro, bas sur DirectInput */
    InitKeybint();

    /* initialisation de la librairie Allegro */
    set_uformat(U_ASCII);  /* pour les accents Latin-1 */
    allegro_init();
    set_config_file(TEO_CONFIG_FILE);
    install_keyboard();
    install_timer();
    if (njoy >= 0)
        install_joystick(JOY_TYPE_AUTODETECT);

    /* dcoration de la fentre */
    set_window_title("Teo - l'mulateur TO8");
    prog_win = win_get_window();
    SetClassLong(prog_win, GCL_HICON,   (LONG) prog_icon);
    SetClassLong(prog_win, GCL_HICONSM, (LONG) prog_icon);

    /* dtection de la prsence de joystick(s) */
    njoy = MIN(TO8_NJOYSTICKS, num_joysticks);

    /* initialisation de l'mulateur */
    printf("Initialisation de l'mulateur...");

    if (to8_Init(TO8_NJOYSTICKS-njoy) == TO8_ERROR)
        ExitMessage(to8_error_msg);

    printf("ok\n");

    /* initialisation du son */
    InitSound(51200);  /* 51200 Hz car 25600 Hz provoque des irrgularits du timer */

    /* initialisation des joysticks */
    InitJoyint(njoy);
    
    /* initialisation du mode graphique */
    switch(gfx_mode)
    {
        case GFX_MODE40:
            if (!InitGraphic(GFX_MODE40, 8, GFX_AUTODETECT_FULLSCREEN, FALSE))
                ExitMessage("Mode graphique non support.");
            break;

        case GFX_MODE80:
            if (!InitGraphic(GFX_MODE80, 8, GFX_AUTODETECT_FULLSCREEN, FALSE))
                ExitMessage("Mode graphique non support.");
            break;

        case GFX_TRUECOLOR:
            if (!InitGraphic(GFX_TRUECOLOR, 15, GFX_AUTODETECT_FULLSCREEN, FALSE))
                if (!InitGraphic(GFX_TRUECOLOR, 16, GFX_AUTODETECT_FULLSCREEN, FALSE))
                    if (!InitGraphic(GFX_TRUECOLOR, 24, GFX_AUTODETECT_FULLSCREEN, FALSE))
                        if (!InitGraphic(GFX_TRUECOLOR, 32, GFX_AUTODETECT_FULLSCREEN, FALSE))
                            ExitMessage("Mode graphique non support.");
            break;

        case GFX_WINDOW:
            alleg_depth = desktop_color_depth();

            switch (alleg_depth)
            {
                case 8:  /* 8bpp */
                default:
                    ExitMessage("Mode graphique non support.");
                    break;

                case 16: /* 15 ou 16bpp */
                    if ( !InitGraphic(GFX_TRUECOLOR, 15, GFX_AUTODETECT_WINDOWED, TRUE) && 
                         !InitGraphic(GFX_TRUECOLOR, 16, GFX_AUTODETECT_WINDOWED, TRUE) )
                            ExitMessage("Mode graphique non support.");
                    gfx_mode = GFX_TRUECOLOR;
                    break;
 
                case 24: /* 24bpp */
                case 32: /* 32bpp */
                    if (!InitGraphic(GFX_TRUECOLOR, alleg_depth, GFX_AUTODETECT_WINDOWED, TRUE))
                        ExitMessage("Mode graphique non support.");
                    gfx_mode = GFX_TRUECOLOR;
                    break;
            }
            windowed_mode = TRUE;
            break;
    }
    
    /* initialisation de l'interface utilisateur Allegro */
    if (!windowed_mode)   
       InitGUI(version_name, FALSE);

    /* installation de la fonction callback de retraage de l'cran ncessaire
       pour les modes fullscreen */
    set_display_switch_callback(SWITCH_IN, RetraceCallback);

    to8_ColdReset();

    if (memo_name[0])
        to8_LoadMemo7(memo_name);

    if (load_state)
        LoadState();

    /* et c'est parti !!! */
    RunTO8(windowed_mode);

    SaveState();

    /* dsinstallation du callback *avant* la sortie du mode graphique */
    remove_display_switch_callback(RetraceCallback);

    /* sortie du mode graphique */
    SetGraphicMode(SHUTDOWN);

    /* sortie de l'mulateur */
    exit(EXIT_SUCCESS);
}

